
#import "IGXMLParser.h"

@interface IGXMLParser()<NSXMLParserDelegate>
{
    NSMutableDictionary* _xmlContents;
    NSMutableArray* _currentLevel;
    NSString* _lastElementName;
    BOOL _needRemoval;
}

@end

@implementation IGXMLParser

+(NSDictionary*)parse:(NSData *)data
{
    IGXMLParser* xp = [[IGXMLParser alloc]init];
    NSXMLParser* parser = [[NSXMLParser alloc]initWithData:data];
    parser.delegate = xp;
    [parser parse];
    
    return xp->_xmlContents;
}

-(void)parserDidStartDocument:(NSXMLParser *)parser
{
    _xmlContents = [[NSMutableDictionary alloc]init];
    _currentLevel = [[NSMutableArray alloc]init];
    [_currentLevel addObject:_xmlContents];
}


-(void)parser:(NSXMLParser *)parser didStartElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName attributes:(NSDictionary *)attributeDict
{
    _lastElementName = elementName;
    
    id obj = [_currentLevel objectAtIndex:_currentLevel.count-1];
    
    if(![obj isKindOfClass:[NSDictionary class]])
        return;
    
    NSMutableDictionary* currentLevelLookup = (NSMutableDictionary*)obj;
    NSMutableDictionary* newLevelLookup = [[NSMutableDictionary alloc]init];
    
    // Look and see if we've already found an element with the same name at this level.
    NSObject* currentVal = currentLevelLookup[elementName];
    if(currentVal != nil)
    {
        // We found an element at this level with the same name.
        // Lets make an array out of it.
        
        
        // First lets see if we already made an array.
        if([currentVal isKindOfClass:[NSArray class]])
        {
            // We already made an array, so just add the new dictionary we created to the array
            NSMutableArray* vals = (NSMutableArray*)currentVal;
            [vals addObject:newLevelLookup];
        }
        else
        {
            // Create the array.
            NSMutableArray* children = [[NSMutableArray alloc]init];
            
            // Add the curren Value first.
            [children addObject: currentVal];
            
            // Add the dictionary we just created.
            [children addObject:newLevelLookup];
            
            [currentLevelLookup setValue:children forKey:elementName];
        }
    }
    else
    {
        // This is a new element, so store the dictionary we just created.
        [currentLevelLookup setValue:newLevelLookup forKey:elementName];
        
        // Remember that this object should not stay on the queue
        _needRemoval = YES;
    }

    // Push the new dictionary on the queue
    [_currentLevel addObject:newLevelLookup];
}

-(void)parser:(NSXMLParser *)parser foundCharacters:(NSString *)string
{
    string = [string stringByTrimmingCharactersInSet:[NSCharacterSet characterSetWithCharactersInString:@"\n "]];
    
    if(string != nil && string.length > 0)
    {
        if(_needRemoval)
            [_currentLevel removeLastObject];
        
        // Store the value in the current level.
        NSMutableDictionary* level = [_currentLevel objectAtIndex:_currentLevel.count-1];
        level[_lastElementName] = string;
        
        _needRemoval = NO;
    }
}


-(void)parser:(NSXMLParser *)parser didEndElement:(NSString *)elementName namespaceURI:(NSString *)namespaceURI qualifiedName:(NSString *)qName
{
    // Element ended, so make sure to pop it off if we haven't done that already.
    if(_needRemoval)
        [_currentLevel removeLastObject];
    
    _needRemoval = YES;
}


@end